// Copyright (C) 2023-present sam/u1f320 (vulpine.solutions)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published
// by the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.
using Coravel.Mailer.Mail;
using Coravel.Mailer.Mail.Interfaces;
using Coravel.Queuing.Interfaces;
using Foxnouns.Backend.Mailables;

namespace Foxnouns.Backend.Services;

public class MailService(ILogger logger, IMailer mailer, IQueue queue, Config config)
{
    private readonly ILogger _logger = logger.ForContext<MailService>();

    public void QueueAccountCreationEmail(string to, string code)
    {
        queue.QueueAsyncTask(async () =>
        {
            await SendEmailAsync(
                to,
                new AccountCreationMailable(
                    config,
                    new AccountCreationMailableView
                    {
                        BaseUrl = config.BaseUrl,
                        To = to,
                        Code = code,
                    }
                )
            );
        });
    }

    public void QueueAddEmailAddressEmail(string to, string code, string username)
    {
        _logger.Debug("Sending add email address email to {ToEmail}", to);
        queue.QueueAsyncTask(async () =>
        {
            await SendEmailAsync(
                to,
                new AddEmailMailable(
                    config,
                    new AddEmailMailableView
                    {
                        BaseUrl = config.BaseUrl,
                        To = to,
                        Code = code,
                        Username = username,
                    }
                )
            );
        });
    }

    public void QueueResetPasswordEmail(string to, string code)
    {
        _logger.Debug("Sending add email address email to {ToEmail}", to);
        queue.QueueAsyncTask(async () =>
        {
            await SendEmailAsync(
                to,
                new ResetPasswordMailable(
                    config,
                    new ResetPasswordMailableView
                    {
                        BaseUrl = config.BaseUrl,
                        To = to,
                        Code = code,
                    }
                )
            );
        });
    }

    public void QueuePasswordChangedEmail(string to)
    {
        _logger.Debug("Sending add email address email to {ToEmail}", to);
        queue.QueueAsyncTask(async () =>
        {
            await SendEmailAsync(
                to,
                new PasswordChangedMailable(
                    config,
                    new PasswordChangedMailableView { BaseUrl = config.BaseUrl, To = to }
                )
            );
        });
    }

    private async Task SendEmailAsync<T>(string to, Mailable<T> mailable)
    {
        try
        {
            await mailer.SendAsync(mailable);
        }
        catch (Exception exc)
        {
            _logger.Error(exc, "Sending email");
        }
    }
}
